
/* Chrysalide - Outil d'analyse de fichiers binaires
 * invoke.c - décompilation des appels de méthode
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include <string.h>


#include "../instruction.h"
#include "../operand.h"
#include "../../../decomp/expr/access.h"
#include "../../../decomp/expr/assign.h"
#include "../../../decomp/expr/call.h"
#include "../../../format/dex/pool.h"


#define get_routine_from_dex_pool(p, i) NULL



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'invoke-direct'.          *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_invoke_direct(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    GDecInstruction *iter;                  /* Boucle de parcours #1       */
    GDecInstruction *list;                  /* Instructions décompilées    */
    size_t count;                           /* Quantité d'opérandes        */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    uint32_t index;                         /* Indice de l'élément visé    */
    GBinRoutine *routine;                   /* Routine visée par l'appel   */
    const char *name;                       /* Chaîne à afficher           */
    GDecInstruction *src;                   /* Source de l'assignation     */
    GDecInstruction *dest;                  /* Destination de l'assignat°  */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    size_t i;                               /* Boucle de parcours #2       */
    GArchOperand *arg;                      /* Argument brut de l'appel    */
    GDecInstruction *reg;                   /* Argument converti           */

    result = NULL;
    iter = NULL;

    list = g_dec_context_get_decomp_instrs(ctx);
    if (list == NULL) return NULL;

    /* Récupération de la méthode */

    count = g_arch_instruction_count_operands(instr);
    operand = g_arch_instruction_get_operand(instr, count - 1);

    index = g_dalvik_pool_operand_get_index(G_DALVIK_POOL_OPERAND(operand));

    routine = get_routine_from_dex_pool(G_DEX_FORMAT(g_dec_context_get_format(ctx)), index);
    if (routine == NULL) return NULL;

    /* Détermination de la routine-cible exacte */

    name = g_binary_routine_get_name(routine);

    if (strcmp(name, "<init>") != 0)
        result = g_routine_call_new(routine);

    else
    {
        src = NULL;    /* Pour gcc... */

        for (iter = g_dec_instruction_get_last(list);
             iter != NULL;
             iter = g_dec_instruction_get_prev_iter(list, iter))
        {
            if (!G_IS_ASSIGN_EXPRESSION(iter)) continue;

            src = g_assign_expression_get_src(G_ASSIGN_EXPRESSION(iter));
            if (!G_IS_ROUTINE_CALL(src)) continue;

            dest = g_assign_expression_get_dest(G_ASSIGN_EXPRESSION(iter));
            /* TODO : vérifier aussi la concordance des registres src && instr */

            break;

        }

        if (iter == NULL) return NULL;

        result = src;

        g_dec_instruction_delete(&list, iter);
        g_dec_context_set_decomp_instrs(ctx, list);

    }

    /* Ajout des arguments */

    operand = g_arch_instruction_get_operand(instr, 0);
    count = g_dalvik_args_count(G_DALVIK_ARGS_OPERAND(operand));

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    for (i = 1; i < count; i++)
    {
        arg = g_dalvik_args_operand_get(G_DALVIK_ARGS_OPERAND(operand), i);
        reg = g_dec_context_convert_register(ctx, arg, false, addr);

        g_routine_call_add_arg(G_ROUTINE_CALL(result), reg);

    }

    return (iter != NULL ? iter : result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'invoke-static'.           *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_invoke_static(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    size_t count;                           /* Quantité d'opérandes        */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    uint32_t index;                         /* Indice de l'élément visé    */
    GBinRoutine *routine;                   /* Routine visée par l'appel   */
    GDecInstruction *call;                  /* Représentation de l'appel   */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    size_t i;                               /* Boucle de parcours #2       */
    GArchOperand *arg;                      /* Argument brut de l'appel    */
    GDecInstruction *reg;                   /* Argument converti           */

    result = NULL;

    /* Récupération de la méthode */

    count = g_arch_instruction_count_operands(instr);
    operand = g_arch_instruction_get_operand(instr, count - 1);

    index = g_dalvik_pool_operand_get_index(G_DALVIK_POOL_OPERAND(operand));

    routine = get_routine_from_dex_pool(G_DEX_FORMAT(g_dec_context_get_format(ctx)), index);
    if (routine == NULL) return NULL;

    call = g_routine_call_new(routine);

    /* Ajout des arguments */

    operand = g_arch_instruction_get_operand(instr, 0);
    count = g_dalvik_args_count(G_DALVIK_ARGS_OPERAND(operand));

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    for (i = 0; i < count; i++)
    {
        arg = g_dalvik_args_operand_get(G_DALVIK_ARGS_OPERAND(operand), i);
        reg = g_dec_context_convert_register(ctx, arg, false, addr);

        g_routine_call_add_arg(G_ROUTINE_CALL(call), reg);

    }

    return call;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'invoke-virtual'.          *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_invoke_virtual(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    size_t count;                           /* Quantité d'opérandes        */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    uint32_t index;                         /* Indice de l'élément visé    */
    GBinRoutine *routine;                   /* Routine visée par l'appel   */
    GDecInstruction *call;                  /* Représentation de l'appel   */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    size_t i;                               /* Boucle de parcours #2       */
    GArchOperand *arg;                      /* Argument brut de l'appel    */
    GDecInstruction *reg;                   /* Argument converti           */

    result = NULL;

    /* Récupération de la méthode */

    count = g_arch_instruction_count_operands(instr);
    operand = g_arch_instruction_get_operand(instr, count - 1);

    index = g_dalvik_pool_operand_get_index(G_DALVIK_POOL_OPERAND(operand));

    routine = get_routine_from_dex_pool(G_DEX_FORMAT(g_dec_context_get_format(ctx)), index);
    if (routine == NULL) return NULL;

    call = g_routine_call_new(routine);

    /* Ajout des arguments */

    operand = g_arch_instruction_get_operand(instr, 0);
    count = g_dalvik_args_count(G_DALVIK_ARGS_OPERAND(operand));

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    for (i = 1; i < count; i++)
    {
        arg = g_dalvik_args_operand_get(G_DALVIK_ARGS_OPERAND(operand), i);
        reg = g_dec_context_convert_register(ctx, arg, false, addr);

        g_routine_call_add_arg(G_ROUTINE_CALL(call), reg);

    }

    /* Appel depuis le propriétaire */

    arg = g_dalvik_args_operand_get(G_DALVIK_ARGS_OPERAND(operand), 0);
    reg = g_dec_context_convert_register(ctx, arg, false, addr);

    result = g_access_expression_new(G_DEC_EXPRESSION(reg), G_DEC_EXPRESSION(call));

    return result;

}
