
/* Chrysalide - Outil d'analyse de fichiers binaires
 * collections.c - enregistrement et la diffusion des collections
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "collections.h"


#include <assert.h>
#include <malloc.h>
#include <pthread.h>


#include "../analysis/db/collection.h"
#include "../analysis/db/protocol.h"
#include "../analysis/db/items/bookmark.h"
#include "../analysis/db/items/comment.h"
#include "../analysis/db/items/move.h"
#include "../analysis/db/items/switcher.h"



/* Mémorisation des types de collection enregistrés */
static GType *_collection_definitions = NULL;
static uint32_t _collection_definitions_count = 0;

/* Verrou pour des accès atomiques */
/* ... */



/******************************************************************************
*                                                                             *
*  Paramètres  : items = type GLib des éléments constituant une collection.   *
*                                                                             *
*  Description : Enregistre un type d'élément à gérer par collection.         *
*                                                                             *
*  Retour      : Identifiant unique attribué "dynamiquement".                 *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

uint32_t register_collection_type(GType items)
{
    uint32_t result;                        /* Identifiant à retourner     */

    /* TODO : lock */

    result = _collection_definitions_count++;

    _collection_definitions = (GType *)realloc(_collection_definitions,
                                               _collection_definitions_count * sizeof(GType));

    _collection_definitions[result] = items;

    /* TODO : unlock */

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Charge les définitions de collections "natives".             *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool load_hard_coded_collection_definitions(void)
{
    uint32_t id;                            /* Identifiant unique retourné */

    /**
     * La liste des chargements doit se faire dans le même ordre que
     * la définition de l'énumération 'DBFeatures' dans le fichier 'protocol.h',
     * afin de garder la correspondance entre les identifiants.
     */

    id = register_collection_type(G_TYPE_BM_COLLECTION);
    assert(id == DBF_BOOKMARKS);

    id = register_collection_type(G_TYPE_COMMENT_COLLECTION);
    assert(id == DBF_COMMENTS);

    id = register_collection_type(G_TYPE_MOVE_COLLECTION);
    assert(id == DBF_MOVES);

    id = register_collection_type(G_TYPE_SWITCHER_COLLECTION);
    assert(id == DBF_DISPLAY_SWITCHERS);

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Décharge toutes les définitions de collections.              *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void unload_collection_definitions(void)
{
    if (_collection_definitions != NULL)
        free(_collection_definitions);

    _collection_definitions = NULL;
    _collection_definitions_count = 0;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Construit un nouvel ensemble de collections.                 *
*                                                                             *
*  Retour      : Liste complète de collections vierges.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GList *create_collections_list(void)
{
    GList *result;                          /* Groupe à retourner          */
    uint32_t i;                             /* Boucle de parcours          */
    GDbCollection *collec;                  /* Nouveau groupe à intégrer   */

    result = NULL;

    for (i = 0; i < _collection_definitions_count; i++)
    {
        collec = g_object_new(_collection_definitions[i], NULL);

        result = g_list_append(result, collec);

    }

    return result;

}
